/* FILE: dspmath.c              Copyright (c), Symmetric Research, 1998
 *
 * This is the C portion of the DSPMATH library.
 *
 *
 * * NOTES ON THE GENERAL OPERATION OF THE DSPMATH LIBRARY:
 *
 * The purpose of the DSPMATH library is to provide an easy way to use
 * the DSP32C board.  When calling the library, all downloading and
 * transferring of the data between the PC and the board is taken care
 * automatically.
 *
 * The operation of the library is divided between this file which runs
 * on the PC and the file dspmath.dsp which runs on the DSP32C board.
 * When a function in this library is called, the board is first checked
 * to make sure the DSPMATH functions have been downloaded, then the
 * user's data is transferred to the board, operated upon, and returned.
 * 
 * The DSP32C functions in dspmath.dsp are downloaded and stay resident
 * on the board throughout the use of the DPSMATH library.  To make sure
 * a download has occurred, a static flag, initialized, is kept in this
 * file and is checked each time a DSPMATH library function is called.
 * Users may begin by simply calling functions in the library, and if
 * the board has not yet been downloaded then it will be.  They may also
 * call init_dspmath themselves at the beginning of their programs and
 * do the initial downloading explicitly.  Init_dspmath returns a 0 or 1
 * according to the success of the download, any may also be used as a
 * presence test for the board.
 *
 * Communications between the PC and DSP32C board are done with two data
 * structures.  An input array in the low memory of the DSP32C is poked
 * with the function selection and pointers to large arrays to work on
 * along with their dimensions.  The arrays themselves are placed on a
 * data stack in the high memory of the DSP32C.  Once the data has been
 * placed on the board, execution is begun.
 *
 * The status of a calculation is monitored through the DSP32C pir
 * register.  When the calculation is begun, this register is cleared
 * off, and when it is done it is set.  If an error occurs during a
 * DSP32C calculation, the pir register is set with an error flag,
 * otherwise a 1 is returned.  The status of the pir register is also
 * saved in the global variable DSPMATH_status, which the user may
 * monitor for possible errors such as divide by zero.
 *
 * Finally, all communication with the DSP32C is done through the
 * DSPTOOLS library dspio.  This avoids using the C runtime, allowing
 * languages other than C to make clean links.
 *
 * 
 * 
 */

#include <stdio.h>
#include <stdlib.h>

#include "dsptools.h"





/* INCLUDE THE DSP32C OBJECT CODE AND SYMBOL TABLE:
 *
 * Dspmath.dsp is assembled to a "hexified" file, so it can be included
 * in this file for downloading, rather than keeping a binary .dso file
 * around.  The hexified version is included in dspmath.dsh, and
 * obtained by assembling dspmath.dsp with the /h option.
 *
 * The symbol table for dspmath.dsp is also included here so this file
 * can know the addresses of the DSP32C functions and data.  The symbol
 * table from dspmath.dsp is obtained in C format by assembling it with
 * the /sc option.  All symbols from dspmath.dsp have a leading DSP_
 * prefix appended to them in the dspmath.dsc.
 *
 *
 */

#include "dspmath.dsh"
#include "dspmath.dsc"





/* ERROR STATUS:
 *
 * The DSPMATH library keeps in track of math errors in the global
 * status variable DSPMATH_status.  After a calculation is completed,
 * the value of this variable may be check to make sure no errors have
 * occured.  The possible return values are:
 *
 *      1 -> SUCCESS            successful calculation
 *      2 -> ERROR_INV          divide by zero
 *      3 -> ERROR_SQRT         square root argument negative
 *      4 -> ERROR_LOG          log of 0 or negative number
 *
 * When an error occurs during the DSP32C's calculations, execution is
 * halted.  The calculation is not carried through to completion.
 *
 *
 */

unsigned DSPMATH_status;





/* HELPER FUNCTIONS:
 *
 * These functions are for controlling DSP32C execution and downloading.
 *
 * The variable data keeps in track of where the last array was pushed
 * in the data_stack area of the DSP32C.
 *
 *
 */

unsigned long beginning_of_dspdata = DSP_data_stack;

static unsigned long data = DSP_data_stack;

FUNCTYPE( INT) init_dspmath( void) {

        static unsigned initialized = 0;


        /* Initialize and download the board if not done yet. */

        if ( !initialized ) {


                /* Select the cpu for DSP_MUL boards. */

                DSP32C_SelectCpu( 0);



                /* Initialize and download. */

                if ( !DSP32C_Init() )

                        return( 0);


                if ( !DSP32C_SrHexLoad( dspmath_dsh) )

                        return( 0);



                /* Successful .... */

                initialized = 1; 

                   beginning_of_dspdata = DSP_data_stack;
                

                }


        /* Point to the top of the PC data stack on the DSP32C. */
        
        data = beginning_of_dspdata;


        return( 1);

}
void execute( void __huge *input, unsigned nbytes) {


        /* Download the input data structure to the DSP32C. */

        DSP32C_Put( input, DSP_input_array, nbytes);



        /* Clear the pir register and execute. */

        DSP32C_SetPir( 0);
        
        DSP32C_Go();

            while ( !DSP32C_Pir() )

                    /* wait for the PIR register to indicate finished ... */ ;

        DSP32C_Halt();



        /* Set the DSPMATH_status parameter. */

        DSPMATH_status = DSP32C_Pir();



}
unsigned long push( void __huge *array, unsigned long nbytes) {

        unsigned long base;
        

        /* Push the array onto the PC data stack on the DSP32C. */
 
        base = data;
        
                data += nbytes;
         

        if ( array )

                DSP32C_Put( array, base, nbytes);


        return( base); /* return the base address */


}       
void pop( unsigned long src, void __huge *dest, unsigned long nbytes) {


        /* Return an array from the PC data stack on the DSP32C. */

        if ( dest )

                DSP32C_Get( src, dest, nbytes);


}






/* DSPMATH LIBRARY FUNCTIONS:
 *
 * These are the functions users may call directl yin the DSPMATH library.
 * 
 * 
 * * DSPMATH.H HAS DATA TYPE DEFINES AND PROTOS:
 *
 * The file dspmath.h is included for its data types defines and to make
 * sure its protos agree the definitions here.  While the data type
 * defines like INPUT and OUTPUT aren't really needed, they make the
 * function protos and definitions shorter and easier to read.
 *
 *
 * * DUMMY ARRAYS NO_PIVOT and etc. INDICATE NO ACTION:
 *
 * A number of library functions may take NULL pointers to indicate
 * special actions.  Gauss is an example, where NULL may be passed to
 * indicate no pivoting.  As an alternative the array NO_PIVOT can also
 * be passed if NULL is not available.
 *
 *
 *
 */

#include "dspmath.h"





/* Helper macro to mov 32 bits of a float into an unsigned long. */

#define mov_float( x)           (*((unsigned long *)&x))



/* Dummy arrays to indicate special actions. */

unsigned long NO_PIVOT[1];

float DEFAULT_TAPER[1], SAME_FILTER[1];

        



/* DSPMATH FUNCTIONS:
 *
 *
 *
 *
 *
 */

FUNCTYPE( VOID) set_waits( unsigned A, unsigned B) {

        struct input_array
                
                { unsigned long func, A, B; } input;
        

        /* Set the wait states for memory paritions A and B. */

        init_dspmath(); input.func = DSP_set_waits;
        
                input.A = A;  /* A = 0 1 2 or 3 */
                input.B = B;  /* B = 0 1 2 or 3 */

                execute( &input, sizeof( input));


}
FUNCTYPE( VOID) add( INPUT x, INPUT y, DIM n, OUTPUT z) {

        struct input_array
                
                { unsigned long func, n, x, y, z; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Add x to y, returning the result in z. */
        
        init_dspmath(); input.func = DSP_add;

                input.n = n;
                input.x = push( x, nbytes);
                input.y = push( y, nbytes);
                input.z = push( NULL, nbytes);

                execute( &input, sizeof( input));

                        pop( input.z, z, nbytes);

}
FUNCTYPE( VOID) sub( INPUT x, INPUT y, DIM n, OUTPUT z) {

        struct input_array
                
                { unsigned long func, n, x, y, z; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Add x to y, returning the result in z. */
        
        init_dspmath(); input.func = DSP_sub;

                input.n = n;
                input.x = push( x, nbytes);
                input.y = push( y, nbytes);
                input.z = push( NULL, nbytes);

                execute( &input, sizeof( input));

                        pop( input.z, z, nbytes);

}
FUNCTYPE( VOID) xpcy( INPUT x, FLOAT c, INPUT y, DIM n, OUTPUT z) {

        struct input_array
                
                { unsigned long func, x, c, y, n, z; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Compute x+c*y, returning the result in z. */
        
        init_dspmath(); input.func = DSP_xpcy;

                input.n = n;
                input.c = mov_float( c);
                input.x = push( x, nbytes);
                input.y = push( y, nbytes);
                input.z = push( NULL, nbytes);

                execute( &input, sizeof( input));

                        pop( input.z, z, nbytes);


}
FUNCTYPE( VOID) scale( INPUT x, DIM n, FLOAT c, OUTPUT y) {

        struct input_array
                
                { unsigned long func, n, x, c, format; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Scale the vector x. */

        init_dspmath(); input.func = DSP_scale;

                input.n = n;
                input.x = push( x, nbytes);
                input.c = mov_float( c);

           input.format = 3;

                execute( &input, sizeof( input));

                        pop( input.x, y, nbytes);


}
FUNCTYPE( VOID) iscale( INPUT x, DIM n, FLOAT c, SINT_ARRAY y, WORD mode) {

        struct input_array
                
                { unsigned long func, n, x, c, format; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Scale the vector x. */

        init_dspmath(); input.func = DSP_scale;

                input.n = n;
                input.x = push( x, nbytes);
                input.c = mov_float( c);

           input.format = mode&0x1;

                execute( &input, sizeof( input));

                        pop( input.x, y, nbytes/2);


}       
FUNCTYPE( VOID) inner( INPUT x, INPUT y, DIM n, OUTPUT result) {

        struct input_array
                
                { unsigned long func, n, x, y, r; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Compute the inner product. */

        init_dspmath(); input.func = DSP_inner;

                input.n = n;
                input.x = push( x, nbytes);
                input.y = push( y, nbytes);
                input.r = push( NULL, nbytes);

                execute( &input, sizeof( input));

                        pop( input.r, result, sizeof( float));


}
FUNCTYPE( VOID) vmult( INPUT x, INPUT y, DIM n, OUTPUT z) {

        struct input_array
                
                { unsigned long func, n, x, y, z; } input;
        
        unsigned long
                
                nbytes = n*sizeof( float);
                
                
        /* Multiply x by y, entry by entry, returning the result in z. */
                
        init_dspmath(); input.func = DSP_vmult;

                input.n = n;
                input.x = push( x, nbytes);
                input.y = push( y, nbytes);
                input.z = push( NULL, nbytes);

                execute( &input, sizeof( input));

                        pop( input.z, z, nbytes);


}
FUNCTYPE( VOID) mmult( INPUT A, INPUT x, DIM n, DIM m, OUTPUT y) {

        struct input_array
                
                { unsigned long func, A, x, y, n, m, n2, nm; } input;

        unsigned long
                
                n2              = n*n,
                n2bytes         = n2*sizeof( float),
                nm              = n*m,
                nmbytes         = nm*sizeof( float);



        /* Compute the Ax for m vectors x. */

        init_dspmath(); input.func = DSP_mmult;
        
                input.A     = push( A, n2bytes);
                input.x     = push( x, nmbytes);
                input.y     = push( NULL, nmbytes);
                input.n     = n;
                input.m     = m;
                input.n2    = n2;
                input.nm    = nm;

                execute( &input, sizeof( input));

                        pop( input.y, y, nmbytes);


}
FUNCTYPE( VOID) AtranA( INPUT A, INPUT b, DIM n, DIM m, OUTPUT ATA, OUTPUT ATb) {

        struct input_array
                
                { unsigned long func, A, b, ATA, ATb, n, m, nm, m2; } input;
        
        unsigned long
                
                nm      = n*m,
                m2      = m*m,
                nbytes  = n*sizeof( float),
                mbytes  = m*sizeof( float),
                nmbytes = nm*sizeof( float),
                m2bytes = m2*sizeof( float);


        /* Compute ATA and ATb. */

        init_dspmath(); input.func = DSP_AtranA;

                input.A    = push( A, nmbytes);
                input.b    = push( b, nbytes);
                input.ATA  = push( NULL, m2bytes);
                input.ATb  = push( NULL, mbytes);
                input.n    = n;
                input.m    = m;
                input.nm   = nm;
                input.m2   = m2;

                execute( &input, sizeof( input));

                        pop( input.ATA, ATA, m2bytes);
                        pop( input.ATb, ATb, mbytes);


}       
FUNCTYPE( VOID) gauss( INPUT A, INPUT b, DIM n, OUTPUT LU, OUTPUT x, LONG_ARRAY p) {

        struct input_array

                { unsigned long func, n, n2, A, b, p; } input;

        unsigned long

                nbytes          = n*sizeof( float),
                n2              = n*n,
                n2bytes         = n2*sizeof( float);



        /* Compute the solution of Ax = b with LU decomposition. */

        init_dspmath(); input.func = DSP_gauss;

                input.n  = n;
                input.n2 = n2;

                input.A = push( A, n2bytes);
                input.b = push( b, nbytes);

                if ( p == NULL || p == (LONG_ARRAY)NO_PIVOT )

                        input.p = 0L;

                else
                        input.p = push( p, nbytes);
                
                
                execute( &input, sizeof( input));

                        pop( input.A, LU, n2bytes);
                        pop( input.b, x, nbytes);

                        if ( p != NULL && p != (LONG_ARRAY)NO_PIVOT )

                                 pop( input.p, p, nbytes);


}
FUNCTYPE( VOID) tridiag( INPUT u, INPUT d, INPUT l, INPUT b, DIM n,

                           OUTPUT LU_u, OUTPUT LU_d, OUTPUT LU_l, OUTPUT x) {

        struct input_array
                
                { unsigned long func, n, u, d, l, b; } input;
        
        unsigned long
                
                nbytes = n*sizeof( float);


        /* Compute the inverse of the tridiagonal matrix. */
                
        init_dspmath(); input.func = DSP_tridiag;
        
                input.n = n;
                input.u = push( u, nbytes);
                input.d = push( d, nbytes);
                input.l = push( l, nbytes);
                input.b = push( b, nbytes);

                execute( &input, sizeof( input));

                        pop( input.u, LU_u, nbytes);
                        pop( input.d, LU_d, nbytes);
                        pop( input.l, LU_l, nbytes);
                        pop( input.b, x, nbytes);

}
FUNCTYPE( VOID) fourier( COMPLEX_INPUT A, DIM n, COMPLEX_OUTPUT FFTA, int isign) {

        struct input_array

                { unsigned long func, A, n, isign; } input;

        unsigned long
                
                nbytes = n*sizeof( COMPLEX);


        /* Compute the Fourier FFT transform. */

        init_dspmath(); input.func = DSP_fourier;

                input.A     = push( A, nbytes);
                input.n     = n;
                input.isign = isign;

                execute( &input, sizeof( input));

                        pop( input.A, FFTA, nbytes);


}
FUNCTYPE( VOID) rfourier( INPUT x, DIM n, OUTPUT FFTx, int isign) {

        struct input_array

                { unsigned long func, x, n, isign; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Compute the real Fourier FFT transform. */

        init_dspmath(); input.func = DSP_rfourier;

                input.x     = push( x, nbytes);
                input.n     = n;
                input.isign = isign;

                execute( &input, sizeof( input));

                        pop( input.x, FFTx, nbytes);


}
FUNCTYPE( VOID) spectrum( INPUT x, INPUT taper, DIM n, OUTPUT xspectrum) {

        struct input_array
                
                { unsigned long func, x, n, isign, taper; } input;
        
        unsigned long
                
                nbytes = n*sizeof( float);



        /* Compute the spectrum of the input. */
        
        init_dspmath(); input.func = DSP_spectrum;
        
                input.x     = push( x, nbytes);
                input.n     = n;
                input.isign = 1;

                if ( taper == NULL || taper == (INPUT)DEFAULT_TAPER )
                        
                        input.taper = 0L;
                
                else
                        input.taper = push( taper, nbytes);


                execute( &input, sizeof( input));
                
                        pop( input.x, xspectrum, nbytes);


}
FUNCTYPE( VOID) convolve( INPUT x1, INPUT x2, INPUT filter, DIM n,

                                                OUTPUT y1, OUTPUT y2) {

        struct input_array

          { unsigned long func, A, n, isign, x1, x2, filter, fflag; } input;

        unsigned long
                
                nbytes    = n*sizeof( float),
                nAbytes   = n*sizeof( COMPLEX);


        /* Compute the convolution of the real time series with the filter. */

        init_dspmath(); input.func = DSP_convolve;

                input.A       = push( NULL, nAbytes);
                input.n       = n;
                input.x1      = push( x1, nbytes);
                input.x2      = push( x2, nbytes);

                if ( filter == NULL || filter == (INPUT)SAME_FILTER ) {
                        
                        input.filter = push( NULL, nbytes);
                        input.fflag = 0;
                        
                        }

                else {
                        input.filter = push( filter, nbytes);
                        input.fflag = 1;
                        
                        }

                execute( &input, sizeof( input));

                        pop( input.x1, y1, nbytes);
                        pop( input.x2, y2, nbytes);


}
FUNCTYPE( VOID) fir( INPUT leader, INPUT x, DIM n, INPUT h, DIM nfilter, OUTPUT y) {
        
        struct input_array
                
                { unsigned long func, x, n, h, nfilter, y; } input;

        
        unsigned long
                
                nbytes          = n*sizeof( float),
                nfilterbytes    = nfilter*sizeof( float);
                

        /* Convolve h with x. */
        
        init_dspmath(); input.func = DSP_fir;
        
                input.x       = push( leader, nfilterbytes);

                                        push( x, nbytes);

                input.n       = n;
                input.h       = push( h, nfilterbytes);
                input.nfilter = nfilter;
                input.y       = push( NULL, nbytes);

                execute( &input, sizeof( input));

                        pop( input.y, y, nbytes);

        
}
FUNCTYPE( VOID) poly_array( INPUT c, DIM nc,

                FLOAT xstart, FLOAT delta, OUTPUT y, DIM n) {


        struct input_array

                { unsigned long func, nc, c, n, y, xstart, delta; } input;

        unsigned long
                
                ncbytes = nc*sizeof( float),
                nbytes  = n*sizeof( float);


        /* Compute the polynomial at n points. */

        init_dspmath(); input.func = DSP_poly_array;
        
                input.nc     = nc;
                input.c      = push( c, ncbytes);
                input.n      = n;
                input.y      = push( NULL, nbytes);
                input.xstart = mov_float( xstart);
                input.delta  = mov_float( delta);

                execute( &input, sizeof( input));

                        pop( input.y, y, nbytes);


}
FUNCTYPE( VOID) varray( unsigned long entry, FLOAT *xstart, FLOAT *delta,

                                                        OUTPUT y, DIM n) {

        struct input_array

                { unsigned long func, entry, n, y, xstart, delta; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Compute a DSP32C function at n equally spaced points. */

        init_dspmath(); input.func = DSP_varray;
        
                input.entry  = entry;
                input.n      = n;
                input.y      = push( NULL, nbytes);
                input.xstart = mov_float( *xstart);
                input.delta  = mov_float( *delta);

                execute( &input, sizeof( input));

                        pop( input.y, y, nbytes);
}

FUNCTYPE( VOID) inv_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n)

        { varray( DSP_inv, &xstart, &delta, y, n); }

FUNCTYPE( VOID) sin_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n)

        { varray( DSP_sin, &xstart, &delta, y, n); }

FUNCTYPE( VOID) cos_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n)

        { varray( DSP_cos, &xstart, &delta, y, n); }

FUNCTYPE( VOID) sqrt_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n)

        { varray( DSP_cos, &xstart, &delta, y, n); }

FUNCTYPE( VOID) log10_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n)

        { varray( DSP_cos, &xstart, &delta, y, n); }


FUNCTYPE( VOID) vfunc( unsigned long entry, INPUT x, DIM n, OUTPUT y) {

        struct input_array
                
                { unsigned long func, entry, x, n; } input;

        unsigned long
                
                nbytes = n*sizeof( float);


        /* Compute a DSP32C function on each element of x[]. */

        init_dspmath(); input.func = DSP_vfunc;
        
                input.entry = entry;
                input.n     = n;
                input.x     = push( x, nbytes);

                execute( &input, sizeof( input));

                        pop( input.x, y, nbytes);
}       

FUNCTYPE( VOID) vsin( INPUT x, DIM n, OUTPUT y)

        { vfunc( DSP_sin, x, n, y); }

FUNCTYPE( VOID) vcos( INPUT x, DIM n, OUTPUT y)

        { vfunc( DSP_cos, x, n, y); }

FUNCTYPE( VOID) vinv( INPUT x, DIM n, OUTPUT y)

        { vfunc( DSP_inv, x, n, y); }

FUNCTYPE( VOID) vsqrt( INPUT x, DIM n, OUTPUT y)

        { vfunc( DSP_sqrt, x, n, y); }

FUNCTYPE( VOID) vlog10( INPUT x, DIM n, OUTPUT y)

        { vfunc( DSP_log10, x, n, y); }


FUNCTYPE( VOID) polar_to_xy( INPUT rtheta, DIM n, OUTPUT xy) {

        struct input_array
                
                { unsigned long func, rtheta, n, xy; } input;
        
        unsigned long
                
                nbytes = 2*n*sizeof( float);
        
        
        /* Convert r theta coordinats to x y. */
        
        init_dspmath(); input.func = DSP_polar_to_xy;
        
                input.rtheta = push( rtheta, nbytes);
                input.n      = n;
                input.xy     = push( NULL, nbytes);
                
                execute( &input, sizeof( input));
                
                        pop( input.xy, xy, nbytes);


}
FUNCTYPE( VOID) interpolate( INPUT array1, DIM n1, OUTPUT array2, DIM n2) {

        struct input_array
                
                { unsigned long func, array1, n1, array2, n2; } input;

        unsigned long

                n1bytes = n1*sizeof( float),
                n2bytes = n2*sizeof( float);


        /* Interpolate array1 onto array2. */

        init_dspmath(); input.func = DSP_interpolate;
        
                input.array1 = push( array1, n1bytes);
                input.n1     = n1;
                input.array2 = push( NULL, n2bytes);
                input.n2     = n2;

                execute( &input, sizeof( input));

                        pop( input.array2, array2, n2bytes);


}
FUNCTYPE( VOID) spline ( INPUT x, INPUT y, OUTPUT h, OUTPUT a, OUTPUT b,

        OUTPUT c, OUTPUT d, OUTPUT xx, OUTPUT yy, DIM n, DIM nn, FLOAT xdiv) {

    struct input_array

      { unsigned long func, x, y, h, a, b, c, d, xx, yy, n, nn, xdiv; } input;
        

        unsigned long
            
            nbytes  =  n*sizeof( float),
            nnbytes = nn*sizeof( float);


        /* Compute the spline coefficients and interpolated points. */
        
        init_dspmath(); input.func = DSP_spline;
        
                input.n    = n;
                input.nn   = nn;
                input.xdiv = mov_float( xdiv);

                input.x    = push( x, nbytes);
                input.y    = push( y, nbytes);
                input.h    = push( NULL, nbytes);
                input.a    = push( NULL, nbytes);
                input.b    = push( NULL, nbytes);
                input.c    = push( NULL, nbytes);
                input.d    = push( NULL, nbytes);
                input.xx   = push( NULL, nnbytes);
                input.yy   = push( NULL, nnbytes);

                execute( &input, sizeof( input) );
        
                        pop( input.h, h, nbytes);
                        pop( input.a, a, nbytes);
                        pop( input.b, b, nbytes);
                        pop( input.c, c, nbytes);
                        pop( input.d, d, nbytes);
                        pop( input.xx, xx, nnbytes);
                        pop( input.yy, yy, nnbytes);

}
