// FILE: a64.cpp                Copyright (c), Symmetric Research, 2003
//
// Source file for DSPA64 specific portion of the Scope program includes
//
//      CA64App
//      CA64Win
//      CHardwareDlg
//      CChannelDlg
//      CAcquireDlg
//
//

#include <io.h>         // needed for _access

#include "a64.h"

extern "C" {
#include "dspa64.h"
}




// Global application object

CA64App APP;


// General pointer to it (defined in scope.cpp)

extern CScopeApp *pAPP;




//***********************************************************************
//
// CLASS CA64App
//
// This class, derived from CScopeApp, contains the DSPA64 specific
// application portion of the Scope data acquisition program.  It works
// in conjunction with its parent class to deal with acquisition, files,
// and the ini.  The CA64Win class takes care of DSPA64 specific
// main window functions.
//
//


BEGIN_MESSAGE_MAP(CA64App, CScopeApp)
        //{{AFX_MSG_MAP(CA64App)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


int CA64App::InitAppData() {
        TRACE0("CA64App::InitAppData\n");

        
        // Initialize common global constants

        CScopeApp::InitAppData();



        // Set specific global application pointer

        pAPP = this;


        
        // Initialize max and min values and allocate scan arrays

        AppTitle.LoadString( IDR_SCOPE );
        AppRev.LoadString(   IDR_REV );

        
        DataPtSize          = sizeof(short);
        ROnsamples          = FALSE;
        
        MaxChannel          = MAXCHANNELS;
        MaxSample           = 0;                // no max
        MinAmpShift         = 0;
        MaxAmpShift         = 16;
        BoardAggSPS         = (float)(XTAL / MIN_NTICKS);
        MinSPS              = BoardAggSPS / 100000.0F;
        MaxSPS              = BoardAggSPS / MaxChannel;
        MaxTriggerThreshold = 32768;            // half of full scale 16 bits
        MaxTriggerDigioLine = -1;               // No hardware support for this

        DefaultColor        = CS_BLUE;
        DefaultNdisplay     = MaxChannel / 4;
        DefaultNsamples     = 128;
        DefaultNblock       = 500 / DefaultNsamples + 1;
        DefaultThreshold    = MaxTriggerThreshold / 2;

        Nplotbuffers = DefaultNblock;

        
        // Initialize additional specific constants

        pDataArray  = NULL;
        ReadyBuffer = 0;
        

        return( 1 );    
}


CScopeWindow* CA64App::NewScopeWindow() {
        TRACE0("CA64App::NewScopeWindow\n");

        CScopeWindow* pWnd = (CScopeWindow*) new CA64Win;

        return( pWnd );
}


int CA64App::ExitInstance() {
        TRACE0("CA64App::ExitInstance\n");

        // Clean up any existing data array

        if (pDataArray)
                { free(pDataArray); pDataArray = NULL; }


        return( CScopeApp::ExitInstance() );
}

int CA64App::ComputeActiveChannels( void ) {
        TRACE0("CA64App::ComputeActiveChannels\n");

        int iret;

        
        // Compute Ini.Nchannels

        iret  = CScopeApp::ComputeActiveChannels( );

                
        // Nchannels may have changed so recompute AcqSPS

        iret &= ComputeSPSValues();

        return( iret );
}

int CA64App::ComputeTimingTicks( void ) {
        TRACE0("CA64App::ComputeTimingTicks\n");

        int newticks, nticks;

        newticks = IniA64.HardJP60+2;

                
        if (newticks < MIN_NTICKS) {
                AfxMessageBox( "Invalid hardware jumper setting used to compute SPS");
                newticks = MIN_NTICKS;
                IniA64.HardJP60 = newticks - 2;
                return( 0 );
                }

        DSPA64_NTimingTicks( newticks, &nticks );
        DSPA64_MaximumRate( &BoardAggSPS );

        return( 1 );
}

int CA64App::ComputeSPSValues( void ) {

        ComputeTimingTicks();
        
        if (Ini.Nchannels <= 0) {
                AfxMessageBox("Invalid Ini.Nchannels used to compute SPS");
                Ini.Nchannels = 1;
                return( 0 );
                }

        DSPA64_NCountToSps( Ini.Nchannels, IniA64.AcqNcount, &Ini.AcqSPS );

        MinSPS = BoardAggSPS / 100000.0F;
        MaxSPS = BoardAggSPS / Ini.Nchannels;

        return( 1 );
}

int CA64App::ComputeNcount( void ) {

        ComputeTimingTicks();
        
        if (Ini.Nchannels <= 0) {
                AfxMessageBox("Invalid Ini.Nchannels used to compute AcqNcount");
                Ini.Nchannels = 1;
                return( 0 );
                }

        // Set AcqNcount based on AcqSPS
        DSPA64_SpsToNCount( Ini.Nchannels, Ini.AcqSPS, &IniA64.AcqNcount );

        // Adjust AcqSPS to match value derived from AcqNcount
        DSPA64_NCountToSps( Ini.Nchannels, IniA64.AcqNcount, &Ini.AcqSPS );

        return( 1 );
}


int CA64App::AllocateDataArray( void ) {
        TRACE0("CA64App::AllocateDataArray\n");

        // Clean up any previous array and allocate new one
        
        if (pDataArray)
                { free(pDataArray); pDataArray = NULL; }

        pDataArray = (short*) malloc(Nbufferbytes);

        

        if ( pDataArray )
                return( 1 );
        else
                return( 0 );
}


int CA64App::TriggerCondition( void ) {
        TRACE0("CA64App::TriggerCondition\n");

        // This function is called by the OnTimer function to decide
        // whether or not the currently acquired buffer of data should
        // be written out.
        //

        // Currently all processing is done by the parent class
        
        return( CScopeApp::TriggerCondition( ) );



        
        // If you want to do any A64 specific processing, remove the
        // return from above and do it here.
        //
        // Because this function is a member of the CA64App class which
        // is derived from the CScopeApp class you have access to all the
        // member variables of both classes.  In particular, you may wish
        // to use
        //
        //      pDataArray      a pointer to the raw data
        //      Ini.Nchannels   the number of channels
        //      Ini.Nsamples    the number of samples per channel

}

int CA64App::TriggerThresholdSetRange( int Chan, long *ThMin, long *ThMax ) {

        TRACE0("CA64App::TriggerThresholdSetRange\n");

        // This function sets the min and max threshold values.
        // DCoffset was computed elsewhere with 16 bit signed data.

        *ThMin = pDCoffset[Chan] - Ini.TrgThreshold;
        if (*ThMin < 0xFFFF8000L)
                *ThMin = 0xFFFF8000L; // min for 16 bit signed data = -32768 
        
        *ThMax = pDCoffset[Chan] + Ini.TrgThreshold;
        if (*ThMax > 0x00007FFFL)
                *ThMax = 0x00007FFFL; // max for 16 bit signed data = 32767

        return( 1 );
}
int CA64App::TriggerThresholdCompare( int Pt, long ThMin, long ThMax ) {
        
        TRACE0("CA64App::TriggerThresholdCompare\n");

        long LongValue;

        // This function compares data point pt with the supplied
        // threshold and returns 1 if the amplitude of the point exceeds
        // the DCoffset corrected threshold in either direction (min or
        // max).

        LongValue = (long)pDataArray[Pt];
        
        if ( LongValue >= ThMax )
                return( 1 );
        else if ( LongValue <= ThMin )
                return( 1 );
        else
                return( 0 );
}
        
int CA64App::TriggerDigioCompare( int Line, int ActiveState ) {

        TRACE0("CA64App::TriggerDigioCompare\n");

        // Digital I/O triggering is not supported on the A64, 
        // so always return true.

        return( 1 );
}
        
int CA64App::FillHdr() {

        // Fill in general header info

        CScopeApp::FillHdr();


        // Fill in A64 specific header info

        strcpy(Hdr.Id, "SR DSPA64");

        Hdr.FilterCoeff = IniA64.AcqCoeff;
        Hdr.FilterScale = IniA64.AcqScale;

        return( 1 );
}


int CA64App::WriteBytes( CFile *outfile ) {
        TRACE0("CA64App::WriteBytes\n");

        // Ensure valid file pointer
        
        if (!outfile)
                return( 0 );


        // Write out A64 specific data bytes
        
        outfile->Write( pDataArray, Nbufferbytes );
        return( 1 );
}


int CA64App::IniVerifyData() {
        TRACE0("CA64App::IniVerifyData\n");

        int  i;
        char SigString[MAXSTR];


        // Verify common data
        
        CScopeApp::IniVerifyData();



        // Hardware parameter verification
        
        if (IniA64.HardBaseio < 0)
                IniA64.HardBaseio = 0x280;

        if ((IniA64.HardJP60 < MIN_NTICKS-2) || (IniA64.HardJP60 > MAX_NTICKS-2)) {
                IniA64.HardJP60 = MIN_NTICKS-2;
                ComputeTimingTicks();
                }

        if (IniA64.HardXtal != (float)XTAL)
                IniA64.HardXtal = (float)XTAL;

                                

        // Channel parameter verification

        for ( i = 0 ;  i < MaxChannel ; i++ ) {

                if (strlen(&Ini.pScanTitle[i*MAXTITLE]) > MAXTITLE) {
                
                        sprintf( SigString, "CH%02d", i );
                        strcpy( &Ini.pScanTitle[i*MAXTITLE], SigString );
                        }
        
                if ((Ini.pScanActive[i] != CHECKED) && (Ini.pScanActive[i] != UNCHECKED))
                        Ini.pScanActive[i] = CHECKED;

                }



        // Acquisition parameter verification
        
        ComputeSPSValues();
        if ( Ini.AcqSPS < (double)MinSPS )
                Ini.AcqSPS = (double)MinSPS;
        else if ( Ini.AcqSPS > (double)MaxSPS )
                Ini.AcqSPS = (double)MaxSPS;
        

        ComputeNcount();
        if (IniA64.AcqNcount < 0)
                IniA64.AcqNcount = 21;
                
        if ((IniA64.AcqNormalize != CHECKED) && (IniA64.AcqNormalize != UNCHECKED))
                IniA64.AcqNormalize = CHECKED;

        if ((IniA64.AcqOverflow != CHECKED) && (IniA64.AcqOverflow != UNCHECKED))
                IniA64.AcqOverflow = CHECKED;

        if ((IniA64.AcqCoeff < 0.0F) || (IniA64.AcqCoeff > 1.0F))
                IniA64.AcqCoeff = 0.8F;

        if ((IniA64.AcqScale < 0.0F) || (IniA64.AcqScale > 1.0F) || (IniA64.AcqNormalize == CHECKED))
                IniA64.AcqScale = 1-IniA64.AcqCoeff;


        return( 1 );
}

int CA64App::IniSetDefaults( void ) {
        TRACE0("CA64App::IniSetDefaults\n");

        // Set common data defaults
        
        CScopeApp::IniSetDefaults();


        // Set hardware defaults
        
        IniA64.HardBaseio = 0x280;
        IniA64.HardJP60   = MIN_NTICKS-2;
        IniA64.HardXtal   = (float)XTAL;


        // Set acquisition defaults
        
        IniA64.AcqNcount    = 21;
        IniA64.AcqNormalize = CHECKED;
        IniA64.AcqOverflow  = CHECKED;
        IniA64.AcqCoeff     = 0.8F;
        IniA64.AcqScale     = 1-IniA64.AcqCoeff;

        Ini.AcqGainLog      = 0;  // 2^^0 is a gain value of 1.0
        ComputeSPSValues();       // set AcqSPS based on AcqNcount and HardJP60
        
        return( 1 );
}


int CA64App::IniSaveCurrent( void ) {
        TRACE0("CA64App::IniSaveCurrent\n");

        // Save common data
        
        CScopeApp::IniSaveCurrent();


        // Save hardware parameters
        
        IniA64Save.HardBaseio = IniA64.HardBaseio;
        IniA64Save.HardJP60   = IniA64.HardJP60;
        IniA64Save.HardXtal   = IniA64.HardXtal;


        // Save acquisition parameters
        // Common acquisition parameters saved in the parent class...
        
        IniA64Save.AcqNcount    = IniA64.AcqNcount;
        IniA64Save.AcqNormalize = IniA64.AcqNormalize;
        IniA64Save.AcqOverflow  = IniA64.AcqOverflow;
        IniA64Save.AcqCoeff     = IniA64.AcqCoeff;
        IniA64Save.AcqScale     = IniA64.AcqScale;

        
        return( 1 );
}
        
int CA64App::IniRestoreCurrent( void ) {
        TRACE0("CA64App::IniRestoreCurrent\n");

        // Restore common data
        
        CScopeApp::IniRestoreCurrent();


        // Restore hardware parameters
        
        IniA64.HardBaseio = IniA64Save.HardBaseio;
        IniA64.HardJP60   = IniA64Save.HardJP60;
        IniA64.HardXtal   = IniA64Save.HardXtal;


        // Restore acquisition parameters
        // Common acquisition parameters restored in the parent class...
        
        IniA64.AcqNcount    = IniA64Save.AcqNcount;
        IniA64.AcqNormalize = IniA64Save.AcqNormalize;
        IniA64.AcqOverflow  = IniA64Save.AcqOverflow;
        IniA64.AcqCoeff     = IniA64Save.AcqCoeff;
        IniA64.AcqScale     = IniA64Save.AcqScale;

        
        return( 1 );
}
        
int CA64App::IniIsChanged( void ) {
        TRACE0("CA64App::IniIsChanged\n");

        // Check common data
        
        if (CScopeApp::IniIsChanged())
                return( 1 );


        // Check hardware parameters
        
        if (IniA64Save.HardBaseio != IniA64.HardBaseio)      return( 1 );
        if (IniA64Save.HardJP60   != IniA64.HardJP60)        return( 1 );
        if (IniA64Save.HardXtal   != IniA64.HardXtal)        return( 1 );


        // Check acquisition parameters
        // Common acquisition parameters checked in the parent class...
        
        if (IniA64Save.AcqNcount    != IniA64.AcqNcount)     return( 1 );
        if (IniA64Save.AcqNormalize != IniA64.AcqNormalize)  return( 1 );
        if (IniA64Save.AcqOverflow  != IniA64.AcqOverflow)   return( 1 );
        if (IniA64Save.AcqCoeff     != IniA64.AcqCoeff)      return( 1 );
        if (IniA64Save.AcqScale     != IniA64.AcqScale)      return( 1 );

        
        return( 0 );
}

int CA64App::IniReadKeyword( void ) {
        TRACE0("CA64App::IniReadKeyword\n");

        int           itmp, iok;
        unsigned long ultmp;
        double        dtmp;


        // Process common keywords in scope parent class

        if ( CScopeApp::IniReadKeyword( ) )
        
                return( 1 );


        
        // Check for hardware parameters

        else if ( parse.KeywordEquals( "Dsp32cBaseio") ) {

                iok = parse.GetHex( &itmp, "illegal Dsp32cBaseio hex value");

                if (iok && itmp >= 0x0)
                        IniA64.HardBaseio = itmp;
                else
                        parse.ErrorIni( "Dsp32cBaseio must be > 0x0");
                }


        else if ( parse.KeywordEquals( "JP60Setting" ) ) {

                iok = parse.GetHex( &itmp, "illegal JP60Setting integer value");

                if ( iok && itmp >= MIN_NTICKS-2  ) {
                        IniA64.HardJP60 = itmp;
                        ComputeTimingTicks();
                        }
                else {
                        sprintf( pAPP->szTemp, "JP60Setting must be > %d",
                               MIN_NTICKS-1);
                        parse.ErrorIni( pAPP->szTemp );
                        }
                }

        // Currently, HardXtal is left fixed at (float)XTAL...
        


        // Check for acquisition parameters

        else if ( parse.KeywordEquals( "Overflow" ) ) {
                
                iok = parse.GetOnOff( &itmp, "Overflow must = {ON,OFF}" );
                
                if (iok)
                        IniA64.AcqOverflow = itmp;
                }

        
        else if ( parse.KeywordEquals( "Normalize" ) ) {
                
                iok = parse.GetOnOff( &itmp, "Normalize must = {ON,OFF}" );
                
                if (iok)
                        IniA64.AcqNormalize = itmp;
                }

        
        else if ( parse.KeywordEquals( "FilterCoeff") ) {

                iok = parse.GetFloat( &dtmp, "FilterCoeff must = float");

                if ( iok && dtmp >= 0.00 && dtmp <= 1.0 )
                        IniA64.AcqCoeff = (float)dtmp;
                else
                        parse.ErrorIni( "FilterCoeff must be in the range [0,1]");
                }


        else if ( parse.KeywordEquals( "FilterScale") ) {

                iok = parse.GetFloat( &dtmp, "FilterScale must = float");

                if ( iok && dtmp >= 0.00 && dtmp <= 1.0 )
                        IniA64.AcqScale = (float)dtmp;
                else
                        parse.ErrorIni( "FilterScale must be in the range [0,1]");
                }



        else if ( parse.KeywordEquals( "ChBufPts" ) ) {

                iok = parse.GetUlong( &ultmp, "illegal ChBufPts integer value");

                if ( iok && ultmp > 0 )
                        Ini.Nsamples = ultmp;
                else
                        parse.ErrorIni( "ChBufPts must be > 0");
                }



        else if ( parse.KeywordEquals( "SamplingRate" ) ) {

                iok = parse.GetFloat( &dtmp, "SamplingRate must = float" );

                // Just check against board aggregate max SPS since the
                // user might be sampling only on 1 channel, another check
                // will be done in VerifyIniData once the number of channels
                // is known.

                if ( iok && dtmp >= (double)MinSPS &&
                            dtmp <= (double)BoardAggSPS ) {
                        Ini.AcqSPS = (float)dtmp;
                        ComputeNcount();
                        }
                else {
                        sprintf( pAPP->szTemp,
                                 "SamplingRate must be %8.2f to %8.2f Hz",
                                 MinSPS, BoardAggSPS );
                        parse.ErrorIni( pAPP->szTemp );
                        }
                }



        else if ( parse.KeywordEquals( "Gain" ) ) {

                iok = parse.GetInt( &itmp, "illegal Gain integer value" );

                if (iok && itmp == 1)
                        Ini.AcqGainLog = 0;
                else
                        parse.ErrorIni( "Gain must = 1" );
                }


        
        
        
        // Unrecognized specific keyword
        
        else 
                return( 0 );


        return( 1 );
}

int CA64App::IniWriteKeyword( FILE *OutFp ) {
        TRACE0("CA64App::IniWriteKeyword\n");

        unsigned long mask;
        int           i;
        char          JP60String[25];
        char         *OverflowString, *NormalizeString;


        // Print common keywords
        
        CScopeApp::IniWriteKeyword( OutFp );

        

        // Set up output strings
        
        OverflowString  = ( IniA64.AcqOverflow  ? "ON" : "OFF" );
        NormalizeString = ( IniA64.AcqNormalize ? "ON" : "OFF" );

       strcpy( JP60String, "; 1 :::::::::::: JP60" );
       mask = 0x0001;
       for ( i = 0 ; i < 12 ; i++ ) {
               if (IniA64.HardJP60 & mask)
                       JP60String[i+4] = 'I';
               else
                       JP60String[i+4] = ':';
               mask = mask << 1;
               }
        

        

        // Print Hardware keywords
        
        fprintf( OutFp, "        Dsp32cBaseio = 0x%03X\n", IniA64.HardBaseio );
        fprintf( OutFp, "         JP60Setting = 0x%03X     %s\n", IniA64.HardJP60, JP60String );
        fprintf( OutFp, "\n" );

        
        // Print acquisition keywords
        // Note: sampling rate and gain are printed by the scope parent class
        
        fprintf( OutFp, "            Overflow = %s\n",    OverflowString );
        fprintf( OutFp, "           Normalize = %s\n",    NormalizeString );
        fprintf( OutFp, "         FilterCoeff = %5.3f\n", IniA64.AcqCoeff );
        fprintf( OutFp, "         FilterScale = %5.3f\n", IniA64.AcqScale );
        fprintf( OutFp, "            ChBufPts = %d\n",    Ini.Nsamples );
        fprintf( OutFp, "\n" );

        
        return( 1 );
}

int CA64App::AtodMeasure( int *ival ) {
        TRACE0("CA64App::AtodMeasure\n");
        
        *ival = 0;
        
        return( 1 );
}

int CA64App::AtodInit() {
        TRACE0("CA64App::AtodInit\n");

        int init;


        // Init DSP and download parms

        if (Ini.DoSysEnable == CHECKED) {

                DSPA64_SetBaseio( IniA64.HardBaseio );
                
                init = DSPA64_Init( Ini.Nchannels, Ini.Nsamples, IniA64.AcqNcount,
                                    IniA64.AcqCoeff, IniA64.AcqScale, Ini.pScanOrder );
                        
                }

        else
                init = 1;


        if (!init)
                ErrString = "Could not initialize the DSPA64 board.\n";



        return( init );
}


int CA64App::AtodStart() {
        TRACE0("CA64App::AtodStart\n");


        // Start A/D execution

        if (Ini.DoSysEnable == CHECKED) {

                DSPA64_Start();
                
                }

        return( 1 );
}


int CA64App::AtodCheckReady() {
        TRACE0("CA64App::AtodCheckReady\n");

        if (Ini.DoSysEnable == CHECKED)
                ReadyBuffer = DSPA64_IsReady();
        else
                ReadyBuffer = 1;

        return( ReadyBuffer );
}


int CA64App::AtodCheckOverflow() {
        TRACE0("CA64App::AtodCheckOverflow\n");

        int overflow;


//WCT:   This may not be a sufficient check, suppose the A/D is
//       running so fast that it skips to the second buffer and
//       not just the next one.  Maybe we really need to have
//       the DSP portion of the code setting a specific overflow
//       flag that we can check.

        // Overflow for the A64 only occurs if the last
        // buffer that was ready is now ready again
        
        if ((Ini.DoSysEnable == CHECKED) && (IniA64.AcqOverflow == CHECKED)) {
                overflow = DSPA64_IsReady();
                if (overflow != ReadyBuffer)
                        overflow = 0;
                }
        else
                overflow = 0;

        return( overflow );
}


int CA64App::AtodGetData() {
        TRACE0("CA64App::AtodGetData\n");

        int iret;
        

        if (Ini.DoSysEnable == CHECKED)
                iret = DSPA64_GetData( ReadyBuffer, pDataArray,
                                        Nbufferbytes);
        else
                iret = 1;
        
        return( iret );
}


int CA64App::AtodDemuxData() {
        TRACE0("CA64App::AtodDemuxData\n");


        int i, j, na, nc, ns, kmux, kdmx, startpt;


        
        // Demux raw data values into the plottable demuxed array

        startpt = Iplotbuff * Ini.Nsamples;


        ns = NsamplesPlot;
        na = Ini.Nsamples;
        nc = Ini.Nchannels;


        for ( j = 0 ; j < nc ; j++ ) {

                pTMPoffset[j] = 0.0;

                for ( i = 0 ; i < na ; i++ ) {

                        kmux = i*nc+j;
                        kdmx = j*ns+i+startpt;

                        pDemuxArray[kdmx] = pDataArray[kmux];

                        pTMPoffset[j] += (double)pDemuxArray[kdmx];
                        
                        }
                }



        // Use the offset computed from the first AD buffer for
        // all parts of the plot buffer
                
        if (ResetDC == TRUE) {

                // If AutoDC is TRUE, then always do ResetDC
                
                if (Ini.PltAutoDC == FALSE)
                        ResetDC = FALSE;
                        
                for ( j = 0 ; j < nc ; j++ )
                        pDCoffset[j] = (long)(pTMPoffset[j] / (double)na);

                }

        
        return( 1 );
}

int CA64App::AtodStop() {
        TRACE0("CA64App::AtodStop\n");

        ReadyBuffer = 0;
        
        DSPA64_Stop();
        
        return( 1 );
}






//***********************************************************************
//
// CLASS CA64Win
//
// This class, derived from CScopeWindow, contains the DSPA64 specific
// main window portion of the Scope data acquisition program.  It primarily
// controls dialogs specific to the DSPA64 while its parent class handles
// the bulk of the graphics drawing, menu operations, and main window
// functions.
//


BEGIN_MESSAGE_MAP( CA64Win, CScopeWindow )
        //{{AFX_MSG_MAP(CA64Win)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CA64Win::CA64Win() 
        :CScopeWindow() {  // parent constructor

        TRACE0("CA64Win::CA64Win\n");
}

CA64Win::~CA64Win() {
        TRACE0("CA64Win::~CA64Win\n");
}


void CA64Win::PostNcDestroy() {
        TRACE0("CA64Win::PostNcDestroy\n");

        // call grandparent function to delete "this"

        CFrameWnd::PostNcDestroy();
}


void CA64Win::OnHardwareDialog() {
        TRACE0("CA64Win::OnHardwareDialog\n");
                
        int          oldJP60, newticks, nticks;
        CHardwareDlg dlg(this);

        
        oldJP60 = APP.IniA64.HardJP60;
        
        if (dlg.DoModal() == IDOK) {

                if (oldJP60 != APP.IniA64.HardJP60) {
                        newticks = APP.IniA64.HardJP60+2;
                        DSPA64_NTimingTicks( newticks, &nticks );
                        if (nticks != newticks) {
                                pAPP->ErrString.Format("Timing ticks = %d\n", nticks);
                                pAPP->ShowError();
                                }
                        APP.ComputeSPSValues();
                        }
                }
                
        controlkey = FALSE;
}

void CA64Win::OnChannelDialog() {
        TRACE0("CA64Win::OnChannelDialog\n");
                
        int oldNdisplay, oldNchannels;

        CChannelDlg dlg(this);

        oldNdisplay  = pAPP->Ini.PltNdisplay;
        oldNchannels = pAPP->Ini.Nchannels;
        
        if (dlg.DoModal() == IDOK) {

                if (oldNchannels < pAPP->Ini.Nchannels )
                        pAPP->IsAllocated = FALSE;

                if (oldNchannels != pAPP->Ini.Nchannels ) {
                        pAPP->ComputeBufferSizes();
                        APP.ComputeSPSValues();
                        }
                
                else if (oldNdisplay != pAPP->Ini.PltNdisplay ) {
                        pAPP->ComputeBufferSizes();
                        }


                CClientDC dc(this);
                ComputeMaxTitleLength( &dc );
                if (!ComputePlotPositions())
                        return;
                
                SetScrollBar( SB_BOTH );

                pAPP->ResetPaint = TRUE;
                        
                        
                Invalidate();                    // cause repaint
                }
                
        controlkey = FALSE;
        
        TRACE0("CA64Win::OnChannelDialog - END\n");
}

void CA64Win::OnAcquireDialog() {
        TRACE0("CA64Win::OnAcquireDialog\n");
        
        CAcquireDlg dlg(this);
        int         oldNdisplay;


        oldNdisplay = pAPP->Ini.PltNdisplay;
        
        if (dlg.DoModal() == IDOK) {
                }

        
        if (oldNdisplay != pAPP->Ini.PltNdisplay ) {
                pAPP->ComputeBufferSizes();
                if (!ComputePlotPositions())
                        return;
                Invalidate();                    // cause repaint
                }
                
        SetScrollBar( SB_BOTH );
                
        controlkey = FALSE;
}








//***********************************************************************
//
// CLASS CHardwareDlg
//
// This class, derived from CScopeDialog, presents and controls the DSPA64
// specific Hardware dialog.  This dialog allows the user to specify the 
// DSPHLF Base IO address and the DSPA64 timer jumper setting.
//


BEGIN_MESSAGE_MAP(CHardwareDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CHardwareDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,        OnEditLostFocus)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT00,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT01,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT02,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT03,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT04,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT05,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT06,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT07,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT08,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT09,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT10,   OnSetfocusBits)
        ON_BN_SETFOCUS(  IDC_CHK_W_BIT11,   OnSetfocusBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT00,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT01,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT02,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT03,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT04,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT05,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT06,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT07,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT08,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT09,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT10,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_CHK_W_BIT11,   OnChangeJumperBits)
        ON_BN_CLICKED(   IDC_BTN_W_CANCEL,  OnCancel)
        ON_BN_CLICKED(   IDC_BTN_W_OK,      OnOK)
        ON_BN_CLICKED(   IDC_BTN_W_HELP,    OnHelp)
        ON_EN_KILLFOCUS( IDC_TXT_W_BASEIO,  OnKillfocusBaseio)
        ON_EN_KILLFOCUS( IDC_TXT_W_MAXRATE, OnKillfocusMaxrate)
        //}}AFX_MSG_MAP
//      ON_CONTROL_RANGE( BN_SETFOCUS,  IDC_CHK_W_BIT00, IDC_CHK_W_BIT11, OnSetfocusBits)
//      ON_CONTROL_RANGE( BN_KILLFOCUS, IDC_CHK_W_BIT00, IDC_CHK_W_BIT11, OnKillfocusBits)
//      ON_CONTROL_RANGE( BN_CLICKED,   IDC_CHK_W_BIT00, IDC_CHK_W_BIT11, OnChangeJumperBits)
END_MESSAGE_MAP()



CHardwareDlg::CHardwareDlg(CWnd* pParent /*=NULL*/)
            : CScopeDialog(CHardwareDlg::IDD, pParent) {

        TRACE0("CHardwareDlg::CHardwareDlg\n");            

        //{{AFX_DATA_INIT(CHardwareDlg)
        m_Baseio = 0;
        m_Maxrate = 0.0f;
        //}}AFX_DATA_INIT
}


void CHardwareDlg::DoDataExchange(CDataExchange* pDX) {
        TRACE0("CHardwareDlg::DoDataExchange\n");
        
        CScopeDialog::DoDataExchange(pDX);
        
        //{{AFX_DATA_MAP(CHardwareDlg)
        DDX_Control(pDX, IDC_TXT_W_MAXRATE, m_TXT_Maxrate);
        DDX_Text(   pDX, IDC_TXT_W_MAXRATE, m_Maxrate);
        //}}AFX_DATA_MAP
}

BOOL CHardwareDlg::OnInitDialog() {
        TRACE0("CHardwareDlg::OnInitDialog\n");

        // Initialize all member variables from application globals

        m_ChkBtn[0]  = IDC_CHK_W_BIT00;
        m_ChkBtn[1]  = IDC_CHK_W_BIT01;
        m_ChkBtn[2]  = IDC_CHK_W_BIT02;
        m_ChkBtn[3]  = IDC_CHK_W_BIT03;
        m_ChkBtn[4]  = IDC_CHK_W_BIT04;
        m_ChkBtn[5]  = IDC_CHK_W_BIT05;
        m_ChkBtn[6]  = IDC_CHK_W_BIT06;
        m_ChkBtn[7]  = IDC_CHK_W_BIT07;
        m_ChkBtn[8]  = IDC_CHK_W_BIT08;
        m_ChkBtn[9]  = IDC_CHK_W_BIT09;
        m_ChkBtn[10] = IDC_CHK_W_BIT10;
        m_ChkBtn[11] = IDC_CHK_W_BIT11;

        
        m_Baseio         = APP.IniA64.HardBaseio;
        m_Jumpers        = APP.IniA64.HardJP60;
        m_Nticks         = m_Jumpers + 2;
        m_NoChange       = TRUE;
        m_LastFocusBtn   = -1;
        m_MinBoardAggSPS = (float)(XTAL / MAX_NTICKS);
        m_MaxBoardAggSPS = (float)(XTAL / MIN_NTICKS);

        ComputeMaxrate();

        SetJumperBits();


        CScopeDialog::OnInitDialog( IDC_BTN_W_OK );



        // Make sure Baseio Address is shown in hex
        
        SetDlgItemHex( IDC_TXT_W_BASEIO, m_Baseio );


        
        // Disable OK button if not in interactive mode

        if ( !pAPP->Ini.DoInteractive ) {
                CWnd *pWnd = GetDlgItem( IDC_BTN_W_OK );
                if (pWnd)
                        pWnd->EnableWindow( FALSE );
                }

        
        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}


void CHardwareDlg::OnCancel() {
        TRACE0("CHardwareDlg::OnCancel\n");
                
        CScopeDialog::OnCancel();
}

void CHardwareDlg::OnOK() {
        TRACE0("CHardwareDlg::OnOK\n");

        if (!CheckFocus())
                return;


        // Verify valid values

        if ( (m_Maxrate < m_MinBoardAggSPS) || (m_Maxrate > m_MaxBoardAggSPS) ) {
                CString msg;
                msg.Format("Max Agg Rate must be between %6.2f and %6.2f",
                        m_MinBoardAggSPS, m_MaxBoardAggSPS );
                AfxMessageBox(msg);
                return;
                }
        
        

        CScopeDialog::OnOK();


        // Update globals
        
        APP.IniA64.HardBaseio = m_Baseio;
        APP.IniA64.HardJP60   = m_Jumpers;
        APP.BoardAggSPS       = m_Maxrate;
}

void CHardwareDlg::OnHelp() {
        TRACE0("CHardwareDlg::OnHelp\n");

        if (!CheckFocus())
                return;

                
        pAPP->ShowHelp(IDR_HELP_HARD_TITLE, IDR_HELP_HARD_TEXT, this);
        
}

LONG CHardwareDlg::OnEditLostFocus( UINT nID, LONG xxx ) {
        TRACE1("CHardwareDlg::OnEditLostFocus (%u)\n", nID);

        BOOL good;
        

        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;
        
        // Validate various edit controls
        
        switch ( nID ) {

                case IDC_TXT_W_BASEIO:

                        good = ValidateAsHex( IDC_TXT_W_BASEIO,
                                                0,
                                                0xFFFF,
                                                &m_Baseio,
                                                "BaseIO address" );

                        if (good)
                                SetDlgItemHex( IDC_TXT_W_BASEIO, m_Baseio );            
                        
                        break;




                case IDC_TXT_W_MAXRATE:
                        
                        if (m_Maxrate > m_MaxBoardAggSPS)
                                m_Maxrate = m_MaxBoardAggSPS;

                        good = ValidateAsFloat( IDC_TXT_W_MAXRATE, 
                                                m_MinBoardAggSPS,
                                                m_MaxBoardAggSPS,
                                                &m_Maxrate,
                                                "Max Aggregate Rate" );

                        if (good)
                                ComputeNticks();

                        
                        break;


                        

                default:
                        good = TRUE;
                        break;
                                                
                        
                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}


void CHardwareDlg::OnSetfocusBits() {
        TRACE0("CHardwareDlg::OnSetfocusBits\n");

        int icurr;
        

        m_LastFocusBtn = -1;
        
        CButton *pBtn = (CButton *)GetFocus();

        if (pBtn) {
                
                icurr = pBtn->GetDlgCtrlID();


                // Make sure current control is a check box
                
                if ((icurr < IDC_CHK_W_BIT00) || (icurr > IDC_CHK_W_BIT11))
                        return;
                

                // Highlight and save id
                
                pBtn->SetState( TRUE );
                
                m_LastFocusBtn = icurr;
                
                }

}

void CHardwareDlg::OnChangeJumperBits() {
        TRACE0("CHardwareDlg::OnChangeJumperBits\n");

        int icurr, ibit, ijump;

        
        CButton *pBtn = (CButton*)GetFocus();

        if (pBtn) {

                icurr = pBtn->GetDlgCtrlID();

                
                // Current control is not a check box
                
                if ( (icurr < IDC_CHK_W_BIT00) | (icurr > IDC_CHK_W_BIT11) )
                        return;


                // Set check state

                if (pBtn->GetCheck() == CHECKED)
                        pBtn->SetCheck( UNCHECKED );
                else
                        pBtn->SetCheck( CHECKED );

                
                // Update jumpers variable

                ibit = icurr - IDC_CHK_W_BIT00;
                
                ijump = m_Jumpers ^= (1<<ibit);

                m_Nticks = m_Jumpers + 2;
        
                ComputeMaxrate();
                
                }

}


void CHardwareDlg::UpdateJumper() {
        TRACE0("CHardwareDlg::UpdateJumper\n");
        
        int i, mask;
        
        m_Jumpers = 0;


        for ( i = 0 ; i < MAXCHKBTN ; i++ ) {

                mask = 1 << i;

                if (IsDlgButtonChecked( m_ChkBtn[i] ))
                        m_Jumpers |= mask;

                }


        m_Nticks = m_Jumpers + 2;
        
        ComputeMaxrate();
}

void CHardwareDlg::OnKillfocusBaseio() {
        TRACE0("CHardwareDlg::OnKillfocusBaseio\n");
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_W_BASEIO, 0 );
}

void CHardwareDlg::OnKillfocusMaxrate() {
        TRACE0("CHardwareDlg::OnKillfocusMaxrate\n");
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_W_MAXRATE, 0 );
}

void CHardwareDlg::ComputeNticks() {
        TRACE0("CHardwareDlg::ComputeNticks\n");

        int   n1, n2;
        float diff1, diff2;
        
        if (m_Maxrate < m_MinBoardAggSPS )
                m_Maxrate = m_MinBoardAggSPS;


        // Compute nticks, round up or down depending on which is closer
        
        n1 = (int)(APP.IniA64.HardXtal / m_Maxrate);

        if (n1 == 0)
                n1 = 1;
        
        n2 = n1+1;

        diff1 = m_Maxrate - (APP.IniA64.HardXtal / n1);
        diff2 = m_Maxrate - (APP.IniA64.HardXtal / n2);

        if (diff1 < 0)
                diff1 = - diff1;
        if (diff2 < 0)
                diff2 = - diff2;

        if (diff1 < diff2)
                m_Nticks = n1;
        else
                m_Nticks = n2;


                
        m_Jumpers = m_Nticks - 2;
        
        SetJumperBits();
}

void CHardwareDlg::ComputeMaxrate() {
        TRACE0("CHardwareDlg::ComputeMaxrate\n");
        
        if (m_Nticks < 1) {
                m_Jumpers = 72;
                m_Nticks  = m_Jumpers + 2;
                SetJumperBits();
                }
        
        m_Maxrate = APP.IniA64.HardXtal / m_Nticks;
        SetDlgItemFloat( IDC_TXT_W_MAXRATE, m_Maxrate );
}


void CHardwareDlg::SetJumperBits() {
        TRACE0("CHardwareDlg::SetJumperBits\n");
        
        int i, mask;

        m_NoChange = TRUE;


        for ( i = 0 ; i < MAXCHKBTN ; i++ ) {

                mask = 1 << i;

                if (m_Jumpers & mask)
                        CheckDlgButton( m_ChkBtn[i], CHECKED );
                else
                        CheckDlgButton( m_ChkBtn[i], UNCHECKED );
                        
                }

        m_NoChange = FALSE;
}





//***********************************************************************
//
// CLASS CChannelDlg
//
// This class, derived from CScopeDialog, presents and controls the DSPA64
// specific Channel dialog.  This dialog allows the user to specify
// the titles and activation status of the channels.
//


BEGIN_MESSAGE_MAP(CChannelDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CChannelDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,     OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_C_OK,      OnOK)
        ON_BN_CLICKED(IDC_BTN_C_CANCEL,  OnCancel)
        ON_BN_CLICKED(IDC_BTN_C_HELP,    OnHelp)
        ON_BN_CLICKED(IDC_BTN_C_BACK,    OnBtnBack)
        ON_BN_CLICKED(IDC_BTN_C_NEXT,    OnBtnNext)
        ON_BN_CLICKED(IDC_BTN_C_ALLON,   OnBtnAllOn)
        ON_BN_CLICKED(IDC_BTN_C_ALLOFF,  OnBtnAllOff)
        ON_BN_CLICKED(IDC_CHK_C_ONOFF,   OnChkOnOff)
        ON_EN_CHANGE(IDC_TXT_C_CURR,     OnChangeTxtCurr)
        ON_EN_KILLFOCUS(IDC_TXT_C_CURR,  OnKillfocusTxtCurr)
        ON_EN_KILLFOCUS(IDC_TXT_C_TITLE, OnKillfocusTxtTitle)
        ON_LBN_SELCHANGE(IDC_LST_C_SCAN, OnSelchangeListScan)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CChannelDlg::CChannelDlg(CWnd* pParent /*=NULL*/)
            : CScopeDialog(CChannelDlg::IDD, pParent) {


        TRACE0("CChannelDlg::CChannelDlg\n");
                 

        //{{AFX_DATA_INIT(CChannelDlg)
        m_CurrChan = 0;
        //}}AFX_DATA_INIT

        

        pTempActive = NULL;
        pTempTitle  = NULL;
        
}

CChannelDlg::~CChannelDlg() {
        TRACE0("CChannelDlg::~CChannelDlg\n");
        
        // Clean up dialog buffers
        
        if (pTempActive)
                { free(pTempActive); pTempActive = NULL; }
        
        if (pTempTitle)
                { free(pTempTitle);  pTempTitle  = NULL; }
}

void CChannelDlg::DoDataExchange(CDataExchange* pDX) {
        TRACE0("CChannelDlg::DoDataExchange\n");
        
        CScopeDialog::DoDataExchange(pDX);
        
        //{{AFX_DATA_MAP(CChannelDlg)
        DDX_Control(pDX, IDC_LST_C_SCAN, m_LSTscan);
        DDX_Text(pDX, IDC_TXT_C_CURR, m_CurrChan);
        //}}AFX_DATA_MAP
}



BOOL CChannelDlg::OnInitDialog() {
        TRACE0("CChannelDlg::OnInitDialog\n");
                
        int i, firstactive;


        // Allocate dialog buffers

        pTempActive = (int*) malloc( pAPP->MaxChannel * sizeof(int) );
        pTempTitle  = (char*)malloc( pAPP->MaxChannel * MAXTITLE *sizeof(char));

                
        if (!pTempActive || !pTempTitle) {
                pAPP->ErrString = "Could not allocate dialog buffers";
                pAPP->ShowError();
                EndDialog( -1 );
                return( FALSE );
                }



        // Initialize all member variables from application globals

        CScopeDialog::OnInitDialog( IDC_BTN_C_OK );

        
        m_Nscan     = 0;
        firstactive = -1;
        
        for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {

                pTempActive[i] = pAPP->Ini.pScanActive[i];
                strcpy(&pTempTitle[i*MAXTITLE],&(pAPP->Ini.pScanTitle[i*MAXTITLE]));

                if (pTempActive[i] == CHECKED) {
                        m_Nscan++;
                        AddItem( i );
                        if (firstactive == -1)
                                firstactive = i;
                        }
                }

        UpdateChannelData( firstactive );
        UpdateInfo();


        // Disable OK button if not in interactive mode

        if ( !pAPP->Ini.DoInteractive ) {
                CWnd *pWnd = GetDlgItem( IDC_BTN_C_OK );
                if (pWnd)
                        pWnd->EnableWindow( FALSE );
                }

                
        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CChannelDlg::OnHelp() {
        TRACE0("CChannelDlg::OnHelp\n");
                
        if (!CheckFocus())
                return;

                
        pAPP->ShowHelp(IDR_HELP_CHAN_TITLE, IDR_HELP_CHAN_TEXT, this);
}

void CChannelDlg::OnCancel() {
        TRACE0("CChannelDlg::OnCancel\n");
                
        CScopeDialog::OnCancel();
}

void CChannelDlg::OnOK() {
        TRACE0("CChannelDlg::OnOK\n");
                
        int ipos, i;
        
        
        if (!CheckFocus())
                return;

        if (m_Nscan == 0) {
                pAPP->ErrString.Format("Must select at least 1 active channel");
                pAPP->ShowError();
                return;
                }
        

        
        
        CScopeDialog::OnOK();


        // Update globals

        ipos = 0;
        
        for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {
                
                pAPP->Ini.pScanActive[i] = pTempActive[i];
                strcpy(&(pAPP->Ini.pScanTitle[i*MAXTITLE]),&pTempTitle[i*MAXTITLE]);
                
                if (pAPP->Ini.pScanActive[i] == CHECKED) {
                        pAPP->Ini.pScanOrder[ipos] = i;
                        ipos++;
                        }
                }

        if (ipos != m_Nscan) {
                pAPP->ErrString.Format("Error in number of active channels");
                pAPP->ShowError();
                }
        
        pAPP->Ini.Nchannels = m_Nscan;
        if (pAPP->Ini.PltNdisplay > pAPP->Ini.Nchannels)
                pAPP->Ini.PltNdisplay = pAPP->Ini.Nchannels;
}

LONG CChannelDlg::OnEditLostFocus( UINT nID, LONG xxx ) {
        TRACE1("CChannelDlg::OnEditLostFocus (%u)\n", nID);

        int  ipos, icurr;
        BOOL good;
        

        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;


                
        // Validate various edit controls
        
        switch ( nID ) {

                case IDC_TXT_C_CURR:

                        icurr = m_CurrChan;

                        // When changing this validation, be sure to also
                        // change the quick validation in OnChkOnOff.

                        good = ValidateAsInt( IDC_TXT_C_CURR,
                                                0,
                                                pAPP->MaxChannel - 1, 
                                                &icurr,
                                                "Current channel");
                        
                        // Always update channel since icurr
                        // will be new or reset from bad
                        
                        UpdateChannelData( icurr );
                        
                        break;



                case IDC_TXT_C_TITLE:
                        
                        GetDlgItemText(IDC_TXT_C_TITLE,
                                &pTempTitle[m_CurrChan*MAXTITLE], MAXTITLE); 
        
                        ipos = FindPos();
                        m_LSTscan.DeleteString(ipos);
                        AddItem( m_CurrChan );
                        ipos = FindPos();
                        m_LSTscan.SetCurSel( ipos );

                        good = TRUE;
                        
                        break;

                        

                default:
                        good = TRUE;
                        break;
                                                
                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}


void CChannelDlg::OnBtnBack() {
        TRACE0("CChannelDlg::OnBtnBack\n");
                
        int icurr;

        if (!CheckFocus())
                return;

                
        icurr = m_CurrChan - 1;
        if (icurr < 0)
                icurr = pAPP->MaxChannel - 1;
        
        UpdateChannelData( icurr );
}

void CChannelDlg::OnBtnNext() {
        TRACE0("CChannelDlg::OnBtnNext\n");
                
        int icurr;

        if (!CheckFocus())
                return;

                
        icurr = m_CurrChan + 1;
        if (icurr > pAPP->MaxChannel - 1)
                icurr = 0;
        
        UpdateChannelData( icurr );
}

void CChannelDlg::OnBtnAllOn() {
        TRACE0("CChannelDlg::OnBtnAllOn\n");
                
        int i;


        if (!CheckFocus())
                return;

                
        m_LSTscan.ResetContent();

        for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {
                AddItem( i );
                pTempActive[i] = CHECKED;
                }
        
        m_Nscan = pAPP->MaxChannel;
        UpdateChannelData(m_CurrChan);
        UpdateInfo();
}

void CChannelDlg::OnBtnAllOff() {
        TRACE0("CChannelDlg::OnBtnAllOff\n");
                
        int i;

        if (!CheckFocus())
                return;

        
        m_LSTscan.ResetContent();

        for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {
                pTempActive[i] = UNCHECKED;
                }

        m_Nscan = 0;
        UpdateChannelData(m_CurrChan);
        UpdateInfo();
}

void CChannelDlg::OnChkOnOff() {
        TRACE1("CChannelDlg::OnChkOnOff (%d)\n", IsDlgButtonChecked(IDC_CHK_C_ONOFF));
                
        int ipos, state, icurr;
        BOOL good;

        // Normally, the current channel is correct when this check box
        // function is called.  But there is a special case that occurs when
        // the user comes here from the current channel text box without
        // tabbing by using the underlined speed keys.  In that case, we
        // get here BEFORE the current channel text box has officially
        // lost focus, been validated, and updated m_CurrChan.

        icurr = GetDlgItemInt( IDC_TXT_C_CURR, &good );
        
        // Handle normal case
        if ( good && (icurr == m_CurrChan) ) {
                if ( IsDlgButtonChecked(IDC_CHK_C_ONOFF) )
                        state = TRUE;
                else
                        state = FALSE;
                }

        // Handle special case by doing a quick validation.  On success,
        // update the channel data and toggle the On/Off check box.  On
        // failure, just return because the current channel text lost
        // focus function will eventually be called and notify the user
        // of the problem.
        
        else {
                if ( good && (icurr >= 0) && (icurr <= pAPP->MaxChannel-1) )
                        UpdateChannelData( icurr );
                else
                        return;
                state = !pTempActive[m_CurrChan];
                }



        GetDlgItem( IDC_LAB_C_TITLE )->EnableWindow( state );
        GetDlgItem( IDC_TXT_C_TITLE )->EnableWindow( state );

        if (state != pTempActive[m_CurrChan]) {

                pTempActive[m_CurrChan] = state;

                if (state) {
                        m_Nscan += 1;
                        AddItem( m_CurrChan );
                        ipos = FindPos();
                        m_LSTscan.SetCurSel( ipos );
                        }
                else {
                        m_Nscan -= 1;
                        ipos = FindPos();
                        m_LSTscan.DeleteString( ipos );
                        }

                UpdateInfo();

                }
}

void CChannelDlg::OnChangeTxtCurr() {
        TRACE0("CChannelDlg::OnChangeTxtCurr\n");
                
        CWnd *pWnd = GetFocus();

        if (pWnd && (pWnd->GetDlgCtrlID() == IDC_TXT_C_CURR) ) 
                GetDlgItem( IDC_LAB_C_CURR )->EnableWindow( FALSE );
}


void CChannelDlg::OnKillfocusTxtCurr() {
        TRACE0("CChannelDlg::OnKillfocusTxtCurr\n");
                
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_C_CURR, 0 );
}

void CChannelDlg::OnKillfocusTxtTitle() {
        TRACE0("CChannelDlg::OnKillfocusTxtTitle\n");
                
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_C_TITLE, 0 );
}

void CChannelDlg::OnSelchangeListScan() {
        TRACE0("CChannelDlg::OnSelchangeListScan\n");
                
        int icurr, ipos;

                
        ipos  = m_LSTscan.GetCurSel();
        TRACE1("CChannelDlg::OnSelchangeListScan - GetCurSel %d\n",ipos);
        if (ipos == LB_ERR) {
                icurr = m_CurrChan;
                TRACE1("CChannelDlg::OnSelchangeListScan - set to m_CurrChan\n",icurr);
                }
        else {
                icurr = m_LSTscan.GetItemData( ipos );
                TRACE1("CChannelDlg::OnSelchangeListScan - GetItemData\n",icurr);
                }
        
        if (icurr != m_CurrChan)
            UpdateChannelData(icurr);

                
        TRACE0("CChannelDlg::OnSelchangeListScan - END\n");
}

void CChannelDlg::UpdateChannelData( int icurr ) {
        TRACE1("CChannelDlg::UpdateChannelData (%d)\n", icurr);
                
        int ipos;

        // Error check, force a valid channel number
        
        if (icurr < 0)
                icurr = 0;

        
        // Save as current channel and update controls accordingly
        
        m_CurrChan = icurr;
        
        SetDlgItemInt(  IDC_TXT_C_CURR,  m_CurrChan );
        CheckDlgButton( IDC_CHK_C_ONOFF, pTempActive[m_CurrChan] );
        SetDlgItemText( IDC_TXT_C_TITLE, &pTempTitle[m_CurrChan*MAXTITLE] );

        GetDlgItem( IDC_LAB_C_TITLE )->EnableWindow( pTempActive[m_CurrChan] );
        GetDlgItem( IDC_TXT_C_TITLE )->EnableWindow( pTempActive[m_CurrChan] );

        ipos = FindPos();
        m_LSTscan.SetCurSel( ipos );

        GetDlgItem( IDC_LAB_C_CURR )->EnableWindow( TRUE );

        TRACE0("CChannelDlg::UpdateChannelData - END\n");
}


void CChannelDlg::UpdateInfo() {
        TRACE0("CChannelDlg::UpdateInfo\n");
                
        CString txt;
        float sps;

        SetDlgItemInt( IDC_LAB_C_NCHAN, m_Nscan );
        DSPA64_NCountToSps( m_Nscan, APP.IniA64.AcqNcount, &sps );
        txt.Format("%6.2f", sps );
        SetDlgItemText( IDC_LAB_C_SPS, txt );
}


void CChannelDlg::AddItem( int ikey ) {
        TRACE1("CChannelDlg::AddItem (%d)\n", ikey);
                
        CString tmpString;
        int     ipos;
        
        tmpString.Format("%02d  ", ikey);
        tmpString += &pTempTitle[ikey*MAXTITLE];
        ipos = m_LSTscan.AddString(tmpString);
        m_LSTscan.SetItemData(ipos,ikey);
}

int CChannelDlg::FindPos() {
        TRACE1("CChannelDlg::FindPos (%d)\n", m_CurrChan);
                
        int ipos, ncount;


        ncount = m_LSTscan.GetCount();

        for ( ipos = 0 ; ipos < ncount ; ipos++ ) {
                if ((int)m_LSTscan.GetItemData(ipos) == m_CurrChan)
                        return( ipos );
                }
        
        return( -1 );
}






//***********************************************************************
//
// CLASS CAcquireDlg
//
// This class, derived from CScopeDialog, presents and controls the DSPA64
// specific Acquisition dialog.  This dialog allows the user to specify
// the sampling rate, the filter coefficients, and the status of the
// DSPHLF.
//


BEGIN_MESSAGE_MAP(CAcquireDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CAcquireDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,        OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_A_OK,         OnOK)
        ON_BN_CLICKED(IDC_BTN_A_CANCEL,     OnCancel)
        ON_BN_CLICKED(IDC_BTN_A_HELP,       OnHelp)
        ON_BN_CLICKED(IDC_BTN_A_CHANNEL,    OnBtnChannels)
        ON_BN_CLICKED(IDC_CHK_A_DSPENABLED, OnChkDSPenable)
        ON_BN_CLICKED(IDC_CHK_A_NORMALIZE,  OnChkNormalize)
        ON_BN_CLICKED(IDC_CHK_A_OVERFLOW,   OnChkOverflow)
        ON_EN_KILLFOCUS(IDC_TXT_A_COEFF,    OnKillfocusTxtCoeff)
        ON_EN_KILLFOCUS(IDC_TXT_A_NCOUNT,   OnKillfocusTxtNcount)
        ON_EN_KILLFOCUS(IDC_TXT_A_SCALE,    OnKillfocusTxtScale)
        ON_EN_KILLFOCUS(IDC_TXT_A_SPS,      OnKillfocusTxtSPS)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()




CAcquireDlg::CAcquireDlg(CWnd* pParent /*=NULL*/)
           : CScopeDialog(CAcquireDlg::IDD, pParent) {

        //{{AFX_DATA_INIT(CAcquireDlg)
        m_DSPenable = CHECKED;
        m_Normalize = CHECKED;
        m_Overflow  = CHECKED;
        m_Coeff     = 0.0f;
        m_Scale     = 0.0f;
        m_SPS       = 0.0f;
        m_Ncount    = 0;
        //}}AFX_DATA_INIT
}


void CAcquireDlg::DoDataExchange(CDataExchange* pDX) {

        CScopeDialog::DoDataExchange(pDX);
        
        //{{AFX_DATA_MAP(CAcquireDlg)
        DDX_Check(pDX, IDC_CHK_A_DSPENABLED, m_DSPenable);
        DDX_Check(pDX, IDC_CHK_A_NORMALIZE,  m_Normalize);
        DDX_Check(pDX, IDC_CHK_A_OVERFLOW,   m_Overflow);
        DDX_Text(pDX,  IDC_TXT_A_COEFF,      m_Coeff);
        DDX_Text(pDX,  IDC_TXT_A_SCALE,      m_Scale);
        DDX_Text(pDX,  IDC_TXT_A_SPS,        m_SPS);
        DDX_Text(pDX,  IDC_TXT_A_NCOUNT,     m_Ncount);
        //}}AFX_DATA_MAP
}


BOOL CAcquireDlg::OnInitDialog() {

        // Initialize all member variables from application globals

        m_SPS       = pAPP->Ini.AcqSPS;

        m_Ncount    = APP.IniA64.AcqNcount;
        m_Normalize = APP.IniA64.AcqNormalize;
        m_Coeff     = APP.IniA64.AcqCoeff;
        m_Scale     = APP.IniA64.AcqScale;
        m_DSPenable = APP.Ini.DoSysEnable;
        m_Overflow  = APP.IniA64.AcqOverflow;

        
        CScopeDialog::OnInitDialog( IDC_BTN_A_OK );

                
        SetDlgItemInt( IDC_LAB_A_NCHANNEL, pAPP->Ini.Nchannels );

        
        // Disable OK button if not in interactive mode

        if ( !pAPP->Ini.DoInteractive ) {
                CWnd *pWnd = GetDlgItem( IDC_BTN_A_OK );
                if (pWnd)
                        pWnd->EnableWindow( FALSE );
                }

        
        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CAcquireDlg::OnCancel() {
        
        CScopeDialog::OnCancel();
}

void CAcquireDlg::OnOK() {
        
        if (!CheckFocus())
                return;

        
        CScopeDialog::OnOK();


        // Update globals

        // Run one last fixup on m_SPS before saving value since it might 
        // have been modified by the automatic DDX text box which only
        // holds 3 digits after the decimal place
        
        DSPA64_NCountToSps( pAPP->Ini.Nchannels, m_Ncount, &m_SPS );
        pAPP->Ini.AcqSPS        = m_SPS;
        
        APP.IniA64.AcqNcount    = m_Ncount;
        APP.IniA64.AcqNormalize = m_Normalize;
        APP.IniA64.AcqCoeff     = m_Coeff;
        APP.IniA64.AcqScale     = m_Scale;
        APP.Ini.DoSysEnable     = m_DSPenable;
        APP.IniA64.AcqOverflow  = m_Overflow;
        
}

void CAcquireDlg::OnHelp() {

        if (!CheckFocus())
                return;

        pAPP->ShowHelp(IDR_HELP_ACQ_TITLE, IDR_HELP_ACQ_TEXT, this);
}

void CAcquireDlg::OnBtnChannels() {

        if (!CheckFocus())
                return;

        
        // Call channel dialog from scope window, restoring focus to
        // channel button when done
        
        CScopeWindow *pWnd = (CScopeWindow *)m_pParentWnd;
        
        pWnd->OnChannelDialog();

        CWnd *pWndItem = GetDlgItem( IDC_BTN_A_CHANNEL );
        if (pWndItem)
                pWndItem->SetFocus( );

        

        // Update Nchannel and Sps controls

        SetDlgItemInt( IDC_LAB_A_NCHANNEL, pAPP->Ini.Nchannels );
        ComputeSampleRate();
}


LONG CAcquireDlg::OnEditLostFocus( UINT nID, LONG xxx ) {
        TRACE1("CAcquireDlg::OnEditLostFocus (%u)\n", nID);

        BOOL good;
        

        if (!DoValidate)
                return( 1 );



        // Don't allow validation when already doing validation

        DoValidate = FALSE;


                
        // Validate various edit controls
        
        switch ( nID ) {

                case IDC_TXT_A_NCOUNT:

                        good = ValidateAsInt( IDC_TXT_A_NCOUNT,
                                                0,
                                                999,
                                                &m_Ncount,
                                                "Oversampling count" );
        
                        if (good) {
                                ComputeSampleRate();
                                }

                        break;




                case IDC_TXT_A_SPS:
                        
                        good = ValidateAsFloat( IDC_TXT_A_SPS,
                                                pAPP->MinSPS,
                                                pAPP->MaxSPS,
                                                &m_SPS,
                                                "Samples/second" );
        
                        if (good) {
                                DSPA64_SpsToNCount( pAPP->Ini.Nchannels, m_SPS,
                                                                &m_Ncount );
                                SetDlgItemInt( IDC_TXT_A_NCOUNT, m_Ncount );
                                ComputeSampleRate();
                                }
                        
                        break;



                        
                case IDC_TXT_A_COEFF:

                        good = ValidateAsFloat( IDC_TXT_A_COEFF,
                                                  0.0F,
                                                  1.0F,
                                                  &m_Coeff,
                                                  "Coefficient" );
        
                        if (good) {
                                if (m_Normalize == CHECKED) 
                                        ComputeScale();
                                }
                                                
                        break;
                        

                        

                case IDC_TXT_A_SCALE:

                        good = ValidateAsFloat( IDC_TXT_A_SCALE,
                                                  0.0F,
                                                  1.0F,
                                                  &m_Scale,
                                                  "Scale" );
        
                        if (good) {
                                if (m_Normalize == CHECKED) {
                                        m_Coeff = 1.0F - m_Scale;
                                        SetDlgItemFloat(IDC_TXT_A_COEFF,m_Coeff);
                                                                
                                        }
                                }

                                                
                        break;


                        
                default:
                        good = TRUE;
                        break;
                                                
                        
                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}


void CAcquireDlg::OnKillfocusTxtNcount() {
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_A_NCOUNT, 0 );
}

void CAcquireDlg::OnKillfocusTxtSPS() {
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_A_SPS, 0 );
}


void CAcquireDlg::ComputeSampleRate() {
        
        BOOL err;
        int  ncnt;

        ncnt = GetDlgItemInt( IDC_TXT_A_NCOUNT, &err );
        DSPA64_NCountToSps( pAPP->Ini.Nchannels, ncnt, &m_SPS );
        SetDlgItemFloat( IDC_TXT_A_SPS, m_SPS );
}

void CAcquireDlg::OnKillfocusTxtCoeff() {
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_A_COEFF, 0 );
}

void CAcquireDlg::OnKillfocusTxtScale() {
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_A_SCALE, 0 );
}

void CAcquireDlg::OnChkNormalize() {

        m_Normalize = IsDlgButtonChecked( IDC_CHK_A_NORMALIZE );

        if (m_Normalize == CHECKED)
                ComputeScale();
}

void CAcquireDlg::ComputeScale() {

        DSPA64_NormalizeScale( m_Coeff, &m_Scale );
        SetDlgItemFloat( IDC_TXT_A_SCALE, m_Scale );
}


void CAcquireDlg::OnChkDSPenable() {

        m_DSPenable = IsDlgButtonChecked( IDC_CHK_A_DSPENABLED );

        if (m_DSPenable == CHECKED)
                GetDlgItem( IDC_CHK_A_OVERFLOW )->EnableWindow( TRUE );
        else
                GetDlgItem( IDC_CHK_A_OVERFLOW )->EnableWindow( FALSE );
                
}

void CAcquireDlg::OnChkOverflow() {

        m_Overflow = IsDlgButtonChecked( IDC_CHK_A_OVERFLOW );
}


